use strict;
use warnings FATAL => 'all';
use Test::More;
use HTML::Zoom;

{
  package BasicPage;
  use HTML::Tags;

  sub show_landing_html {
    as_html(\&landing, (
      title => "Welcome to the Demo Home",
      site_version => 10,
      new_user_link => 'create_user.html',
    ));
  }

  sub show_landing_events {
    as_events(\&landing, (
      title => "Welcome to the Demo Home",
      site_version => 10,
      new_user_link => 'create_user.html',
    ));
  }

  sub layout {
    my (%data) = @_;
    \'<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" ',
    \'"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">',
    <html>,
      <head>,
        <!-- here is a comment -->,
        <title>, ($data{title} || 'Hello World'), </title>,
      </head>,
      <body>,
        @{$data{content}},
      </body>,
    </html>;
  }

  sub landing {
    my (%data) = @_;
    <p>, "Hi, I'm version: $data{site_version}", </p>,
    <p>, "Here's some interesting things about me", </p>,
    <img src="smilyface.png" alt="smiles" />,
    <ul>,
      <li>, <a href="/user">, "My Users", </li>,
      <li>, <a href="/user/$data{new_user_link}">, "Create", "New User", </li>,
    </ul>;
  }

  sub process_templates {
    my ($templates, %data) = @_;
    for my $template(@$templates) {
      my @processed = $template->(%data);
      $data{content} = \@processed;
    }
    return @{$data{content}};
  }

  sub as_html {
    my ($template, %data) = @_;
    my @content = process_templates([$template, \&layout], %data);
    return join '', HTML::Tags::to_html_string(@content);
  }

  sub as_events {
    my ($template, %data) = @_;
    my @content = process_templates([$template, \&layout], %data);
    return [HTML::Tags::to_zoom_events(@content)];
  }
}

ok my $html = BasicPage->show_landing_html;
ok my $zoom = HTML::Zoom->from_html($html);
ok my $events = BasicPage->show_landing_events;

is_deeply $zoom->to_events, $events,
  'Made HZoom events from XMLTags';

#use Data::Dump 'dump';
#warn dump $html;
#warn dump $zoom->to_events;
#warn "======================";
#warn dump $events;

done_testing();
